# By Justin Walgran
# Copyright (c) 2012 Azavea, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import unittest
import os
import shutil
import tempfile
from helpers import create_test_file_with_content
from blend import JSLintAnalyzer, Resource

class TestJSLintAnalyzer(unittest.TestCase):

    def setUp(self):
        self.test_env_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.test_env_dir)

    def make_a_js_file_with_no_lint(self):
        test_file_path = os.path.join(self.test_env_dir, 'test.js')
        create_test_file_with_content(test_file_path, 'var answer = 42;')
        return test_file_path

    def make_a_js_file_with_lint(self):
        test_file_path = os.path.join(self.test_env_dir, 'test.js')
        create_test_file_with_content(test_file_path, 'answer = 42;') # missing 'var'
        return test_file_path

    def test_analysis_fails_when_lib_dir_is_not_found(self):
        jsl = JSLintAnalyzer(lib_path='/some/invalid/path', use_nodejs_if_available=False)
        test_resource = Resource(self.make_a_js_file_with_no_lint())
        analysis = jsl.analyze(test_resource)
        self.assertFalse(analysis.good, 'Expected the analysis to be bad since the tools could not be found')
        self.assertEquals(1, len(analysis.errors))
        self.assertEquals('No suitable JSLint runner (cscript.exe, node.js or rhino) could be found.', analysis.errors[0])

    def test_analysis_succeeds_for_resource_with_no_lint(self):
        jsl = JSLintAnalyzer()
        test_resource = Resource(self.make_a_js_file_with_no_lint())
        analysis = jsl.analyze(test_resource)
        self.assertTrue(analysis.good)

    def test_analysis_fails_for_resource_with_lint(self):
        jsl = JSLintAnalyzer()
        test_resource = Resource(self.make_a_js_file_with_lint())
        analysis = jsl.analyze(test_resource)
        self.assertFalse(analysis.good)
        self.assertEquals(1, len(analysis.errors))


# By Justin Walgran
# Copyright (c) 2012 Azavea, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import unittest
import os
import shutil
import tempfile
from helpers import create_test_file_with_content
from blend import SizeAnalyzer, Resource

class TestSizeAnalyzer(unittest.TestCase):

    def setUp(self):
        self.analyzer = SizeAnalyzer()
        self.test_env_dir = tempfile.mkdtemp()
        self.test_file_path = os.path.join(self.test_env_dir, 'test_file.txt')
        create_test_file_with_content(self.test_file_path, 'some\ttext\non two lines')
        self.resource = Resource(self.test_file_path)

    def tearDown(self):
        shutil.rmtree(self.test_env_dir)

    def test_analyze_method_produces_an_analysis_instance(self):
        analysis = self.analyzer.analyze(self.resource)
        self.assertIsNotNone(analysis)

    def test_analysis_is_always_good(self):
        analysis = self.analyzer.analyze(self.resource)
        self.assertTrue(analysis.good)

    def test_analysis_has_no_warnings_no_errors_and_one_message(self):
        analysis = self.analyzer.analyze(self.resource)
        self.assertIsNone(analysis.warnings)
        self.assertIsNone(analysis.errors)
        self.assertIsNotNone(analysis.messages)
        self.assertEqual(1, len(analysis.messages))

    def test_analysis_message_contatins_char_count_line_count_and_file_size(self):
        analysis = self.analyzer.analyze(self.resource)
        self.assertEqual('%s: %d characters in %d lines for %d bytes' % (self.resource.path_to_file, 21, 2, 22), analysis.messages[0])

    def test_analysis_converted_to_string_is_the_single_message(self):
        analysis = self.analyzer.analyze(self.resource)
        self.assertEqual(analysis.messages[0], str(analysis))



# By Justin Walgran
# Copyright (c) 2012 Azavea, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import unittest
import inspect

from blend import *

class TestBlend(unittest.TestCase):
    """Assert that the blend module contains a set of expected classes and methods."""
    def test_blend_has_a_resource_class(self):
        inspect.isclass(Resource)
        
    def test_blend_has_a_requirement_class(self):
        inspect.isclass(Requirement)

    def test_blend_has_a_paths_class(self):
        inspect.isclass(Paths)

    def test_blend_has_an_application_class(self):
        inspect.isclass(Application)

    def test_blend_has_an_analyzer_class(self):
        inspect.isclass(Analyzer)

    def test_blend_has_an_analysis_class(self):
        inspect.isclass(Analysis)

    def test_blend_has_a_size_analyzer_class(self):
        inspect.isclass(SizeAnalyzer)

    def test_blend_has_a_configuration_class(self):
        inspect.isclass(Configuration)

    def test_blend_has_a_JSLintAnalyzerClass(self):
        inspect.isclass(JSLintAnalyzer)

    def test_blend_has_a_minifier_class(self):
        inspect.isclass(Minifier)

    def test_blend_has_a_Result_class(self):
        inspect.isclass(Result)

    def test_blend_has_a_minification_class(self):
        inspect.isclass(Minification)

    def test_blend_has_a_YUICompressorMinifier_class(self):
        inspect.isclass(YUICompressorMinifier)



# By Justin Walgran
# Copyright (c) 2012 Azavea, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import unittest
import tempfile

from blend import Resource, Paths
from blend.Requirement import RequirementNotSatisfiedException
import shutil
import os
import helpers

from blend import Result

class TestResult(unittest.TestCase):
    """Asserts that the properties and methods of the Result class behave correctly."""

    def setUp(self):
        self.result = Result()

    def tearDown(self):
        pass

    def test_adding_none_to_messages_does_not_create_a_message(self):
        self.result.add_message(None)
        self.result.add_warning("warning")
        self.result.add_error("error")
        self.assertIsNone(self.result.messages, "Expected adding a None message to not add an item to Result.messages")

    def test_adding_none_to_warnings_does_not_create_a_warning(self):
        self.result.add_message("message")
        self.result.add_warning(None)
        self.result.add_error("error")
        self.assertIsNone(self.result.warnings, "Expected adding a None warning to not add an item to Result.warnings")

    def test_adding_none_to_messages_does_not_create_a_message(self):
        self.result.add_message("message")
        self.result.add_warning("warning")
        self.result.add_error(None)
        self.assertIsNone(self.result.errors, "Expected adding a None message to not add an item to Result.messages")

    def test_errors_warnings_and_messages_as_string_with_one_of_each(self):
        self.result.add_message("message")
        self.result.add_warning("warning")
        self.result.add_error("error")
        self.assertEqual("error\nwarning\nmessage", self.result.errors_warnings_and_messages_as_string)

    def test_errors_warnings_and_messages_as_string_with_message_and_warning(self):
        self.result.add_message("message")
        self.result.add_warning("warning")
        self.assertEqual("warning\nmessage", self.result.errors_warnings_and_messages_as_string)


# By Justin Walgran
# Copyright (c) 2012 Azavea, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import unittest
import os

from blend import Paths

class TestPaths(unittest.TestCase):

    def test_paths_has_a_search_paths_property(self):
        env = Paths()
        self.assertNotEqual(None, env.search_paths)

    def test_default_path_property_value_is_an_array_containing_cwd(self):
        env = Paths()
        self.assertEqual([os.getcwd()], env.search_paths)

    def test_paths_can_be_created_with_search_paths(self):
        env = Paths('/tmp/test', '/tmp/lib')
        self.assertEqual(['/tmp/test', '/tmp/lib', os.getcwd()], env.search_paths)

    def test_cwd_can_be_omitted_from_paths(self):
        env = Paths('/tmp/test', '/tmp/lib', include_cwd = False)
        self.assertEqual(['/tmp/test', '/tmp/lib'], env.search_paths)

    def test_creating_with_none_results_in_cwd_as_the_only_path(self):
        env = Paths(None)
        self.assertEqual([os.getcwd()], env.search_paths)



#!/usr/bin/env python

# By Justin Walgran
# Copyright (c) 2012 Azavea, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import unittest

import os
import sys
import inspect
import pkgutil

if __name__ == "__main__":
    sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

    from blend import test

    suite = unittest.TestSuite()

    for importer, modname, ispkg in pkgutil.iter_modules(test.__path__):
        module = __import__('blend.test.' + modname, globals(), locals(), [modname])
        for name, class_obj in inspect.getmembers(module):
            if inspect.isclass(class_obj):
                suite.addTest(unittest.makeSuite(class_obj))

    results = unittest.TextTestRunner(verbosity=2).run(suite)
    # Return the failure count as the exit code of the process. No failures = clean exit.
    sys.exit(len(results.failures))




import os

def first_file_name_in_path_matching_regex(path, regex):
    if os.path.exists(path):
        for file_name in os.listdir(path):
            if regex.match(file_name):
                return os.path.join(path, file_name)
    return None


# By Justin Walgran
# Copyright (c) 2012 Azavea, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import os

class Paths:
    """
    A container for all the paths that should be searched when attempting to locate
    required resources.
    """

    def __init__(self, *args, **kwargs):
        """
        Arguments:
        *args -- Directory paths which will be recursively searched for resources. Order
        is important because the first matching resource will be returned from a search.
        Paths will be searched in order from left to right.
        **kwargs -- Configuration options:
            include_cwd -- Boolean (default value is True) defining whether the current
            working directory will be searched after any other specified directories.
        """
        self._search_paths = []
        for arg in args:
            if arg is not None:
                self._search_paths.append(arg)
        if 'include_cwd' not in kwargs or ('include_cwd' in kwargs and kwargs['include_cwd']):
            self._search_paths.append(os.getcwd())

        if 'output_path' in kwargs:
            self._output_path = kwargs['output_path']
        else:
            self._output_path = os.path.join(os.getcwd(), 'output')

    @property
    def search_paths(self):
        """
        An ordered list of paths in which resources will be located.
        """
        return self._search_paths

    @property
    def output_path(self):
        """
        An ordered list of paths in which resources will be located.
        """
        return self._output_path


# By Justin Walgran
# Copyright (c) 2012 Azavea, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
from fnmatch import fnmatch

import os
import json
import types

from Analyzer import Analyzer
from Minifier import Minifier

class Configuration():

    def __init__(self, config_file_path=None):
        self.analyzers = None
        self.analyzer_skip_lists = None
        self.minifiers = None
        if config_file_path is not None:
            if not os.path.exists(config_file_path):
                raise Exception('Config file "%s" does not exist or is not accessible.' % config_file_path)
            f = open(config_file_path, 'r')
            try:
                configuration_dict = json.load(f)
            finally:
                f.close()

            if 'analyzers' in configuration_dict:
                analyzers_dict = configuration_dict['analyzers']
                for file_type in analyzers_dict.iterkeys():
                    analyzer_list = analyzers_dict[file_type]
                    for analyzer_dict in analyzer_list:
                        analyzer_class = self._get_class(analyzer_dict['name'])
                        if 'skip_list' in analyzer_dict:
                            self.add_analyzer_for_file_type(analyzer_class(), file_type, analyzer_dict['skip_list'])
                        else:
                            self.add_analyzer_for_file_type(analyzer_class(), file_type)

            if 'minifiers' in configuration_dict:
                minifiers_dict = configuration_dict['minifiers']
                for file_type in minifiers_dict.iterkeys():
                    minifier_dict = minifiers_dict[file_type]
                    minifier_class =  self._get_class(minifier_dict['name'])
                    self.set_minifier_for_file_type(minifier_class(), file_type)

    def _get_class(self, kls):
        parts = kls.split('.')
        module = ".".join(parts[:-1])
        m = __import__( module )
        for comp in parts[1:]:
            m = getattr(m, comp)
        return m

    def add_analyzer_for_file_type(self, analyzer, file_type, skip_list=None):
        if not isinstance(analyzer, Analyzer):
            raise Exception('You must pass and Analyzer instance')
        if skip_list is not None and isinstance(skip_list, types.StringTypes):
            raise Exception('You must pass a list to the skip_list parameter')
        if self.analyzers is None:
            self.analyzers = {}
        if file_type not in self.analyzers:
            self.analyzers[file_type] = []
        if analyzer not in self.analyzers[file_type]:
            self.analyzers[file_type].append(analyzer)
            if skip_list:
                if self.analyzer_skip_lists is None:
                    self.analyzer_skip_lists = {}
                if analyzer not in self.analyzer_skip_lists:
                    self.analyzer_skip_lists[analyzer] = {}
                self.analyzer_skip_lists[analyzer][file_type] = skip_list

    def get_analyzers_for_resource(self, resource):
        if self.analyzers is None: return None

        if resource.file_type in self.analyzers:
            if self.analyzer_skip_lists:
                analyzers = []
                for analyzer in self.analyzers[resource.file_type]:
                    if analyzer in self.analyzer_skip_lists:
                        skip_lists_for_file_type = self.analyzer_skip_lists[analyzer]
                        if resource.file_type in skip_lists_for_file_type:
                            skip_list = skip_lists_for_file_type[resource.file_type]
                            skip = False
                            for pattern in skip_list:
                                if  os.path.isabs(pattern):
                                    full_pattern = pattern
                                else:
                                    full_pattern = os.path.join(os.getcwd(), pattern)
                                if fnmatch(resource.path_to_file, full_pattern):
                                    skip = True
                                    break
                            if not skip:
                                analyzers.append(analyzer)
                        else:
                            analyzers.append(analyzer)
                    else:
                        analyzers.append(analyzer)
                if len(analyzers) > 0:
                    return analyzers
                else:
                    return None
            else:
                return self.analyzers[resource.file_type]
        else:
            return None

    def set_minifier_for_file_type(self, minifier, file_type):
        if not isinstance(minifier, Minifier):
            raise Exception('You must pass a Minifier instance')
        if self.minifiers is None:
            self.minifiers = {}
        self.minifiers[file_type] = minifier

    def get_minifier_for_file_type(self, file_type):
        if self.minifiers is None:
            return None

        if file_type in self.minifiers:
            return self.minifiers[file_type]
        else:
            return None




# By Justin Walgran
# Copyright (c) 2012 Azavea, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import os
import re
import subprocess

from Minifier import Minifier
from Minification import Minification
from helpers import first_file_name_in_path_matching_regex

class YUICompressorMinifier(Minifier):

    def __init__(self, lib_path=None):
        self._yuic_proc_args = None
        self._module_path = os.path.dirname(os.path.realpath(__file__))
        self._lib_path = lib_path or os.path.join(self._module_path, 'lib')
        self._yuic_jar_regex = re.compile(r'^yuicompressor.*\.jar$')
        self._yuic_jar_file_path = first_file_name_in_path_matching_regex(self._lib_path, self._yuic_jar_regex)

    def minify(self, resource):
        minification = Minifier.minify(self, resource)
        if self._yuic_jar_file_path is None:
            minification.mark_as_bad()
            minification.add_error('A YUI Compressor .jar file could not be found in %s.' % self._lib_path)
            return minification

        if resource.minified:
            minification.set_content(resource.content)
            minification.mark_as_good()
            minification.add_message('The resource %s is already minified.' % resource.path_to_file)
        else:
            yuic_proc_args = ["java", "-jar", self._yuic_jar_file_path, resource.path_to_file]
            yuic_proc = subprocess.Popen(yuic_proc_args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            yuic_output = yuic_proc.communicate()

            if yuic_proc.returncode == 0:
                minification.set_content(yuic_output[0])
                minification.mark_as_good()
            else:
                minification.mark_as_bad()
                minification.add_error("The YUI Compressor had non-zero exit code: " + str(yuic_proc.returncode) +
                    "\n    " + str(yuic_proc_args) +
                    "\n    " + yuic_output[1])

        return minification


# By Justin Walgran
# Copyright (c) 2012 Azavea, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import unittest
import tempfile

from blend import Resource, Paths
from blend.Requirement import RequirementNotSatisfiedException
import shutil
import os
import helpers


class TestResource(unittest.TestCase):
    """Asserts that the properties and methods of the Resource class behave correctly."""

    def setUp(self):
        self.test_env_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.test_env_dir)
        helpers.clean_output()

    def test_resource_must_be_created_with_path_to_file(self):
        self.assertRaises(Exception, Resource)

    def test_resource_must_be_created_with_a_non_none_path_to_file(self):
        self.assertRaises(Exception, Resource, None)

    def test_resource_must_be_created_with_a_string_argument(self):
        self.assertRaises(Exception, Resource, 1)

    def test_resource_has_path_to_file_property_set_on_instantiation(self):
        test_file_name = 'some/file/name.txt'
        resource = Resource(test_file_name)
        self.assertEqual(test_file_name, resource.path_to_file)

    def test_resource_has_a_file_type_property_with_a_default_value_of_unknown(self):
        test_file_name = 'someFile'
        resource = Resource(test_file_name)
        self.assertEqual('unknown', resource.file_type)

    def test_resource_has_a_size_property(self):
        test_env_dir = tempfile.mkdtemp()
        test_file_path = os.path.join(test_env_dir, 'test_file.txt')
        helpers.create_test_file_with_content(test_file_path, 'some\ttext\non two lines')
        resource = Resource(test_file_path)
        self.assertEqual(22, resource.size)
        shutil.rmtree(test_env_dir)

    def test_resource_has_an_extension_property(self):
        test_file_names_and_expected_extensions = [
            ('/var/someFile.js',       'js'),
            ('someFile.JS',            'js'),
            ('noExtension',            ''),
            ('someFile.SomECraZytype', 'somecrazytype')]
        for test_file_path, expected_extension in test_file_names_and_expected_extensions:
            resource = Resource(test_file_path)
            self.assertEqual(expected_extension, resource.extension, 'Expected the extension of "' +
                test_file_path + '" to be "' + expected_extension + '"')

    def test_resource_detects_file_type_by_extension(self):
        test_file_paths_and_expected_file_types = [
            ('file.someCrazyThing', 'unknown'),
            ('c:\\file.js',         'javascript'),
            ('file.Js',             'javascript'),
            ('file.JS',             'javascript'),
            ('file.awesome.js',     'javascript'),
            ('file.JavaScript',     'javascript'),
            ('file.css',            'css'),
            ('FILE.CSS',            'css')]
        for test_file_path, expected_file_type in test_file_paths_and_expected_file_types:
            resource = Resource(test_file_path)
            self.assertEqual(expected_file_type, resource.file_type, 'Expected "' + test_file_path +
                '" to be detected as "' + expected_file_type + '"')

    def test_resource_has_a_base_name_property(self):
        test_file_paths_and_expected_base_names = [
            ('/usr/local/file.js',       'file'),
            ('FILE.JS',                  'file'),
            ('some-Plugin-2.3.2-min.js', 'some-plugin'),
            ('jQuery-1.2.3.js',          'jquery')]
        for test_file_path, expected_base_name in test_file_paths_and_expected_base_names:
            resource = Resource(test_file_path)
            self.assertEqual(expected_base_name, resource.base_name, 'Expected the base_name of "' +
                test_file_path + '" to be "'+ expected_base_name + '" and not "' + resource.base_name + '"')

    def test_find_all_javascript_resources_in_the_paths(self):
        paths_to_test_files = [os.path.join(self.test_env_dir, 'test.js'), os.path.join(self.test_env_dir, 'test.css'),
            os.path.join(self.test_env_dir, 'test.html')]
        helpers.create_test_files(paths_to_test_files)
        resources = Resource.find_all_of_type_in_paths('javascript', Paths(self.test_env_dir, include_cwd=False))
        helpers.clean_up_test_files(paths_to_test_files)
        self.assertEquals(1, len(resources), 'One and only one javascript file should be found')

    def test_find_all_javascript_resources_in_paths(self):
        paths_to_test_files = [os.path.join(self.test_env_dir, 'subdir', 'test.js'),
            os.path.join(self.test_env_dir, 'test.css'), os.path.join(self.test_env_dir, 'test.html')]
        test_env = Paths(self.test_env_dir, include_cwd=False)
        helpers.create_test_files(paths_to_test_files)
        resources = Resource.find_all_of_type_in_paths('javascript', test_env)
        helpers.clean_up_test_files(paths_to_test_files)
        self.assertEquals(1, len(resources), 'One and only one javascript file should be found')
        self.assertEqual(os.path.join(self.test_env_dir, 'subdir', 'test.js'),
            resources[0].path_to_file)

    def test_search_for_javascript_resources_in_paths_without_any_search_paths_returns_none(self):
        test_env = Paths(self.test_env_dir, include_cwd=False)
        path_to_test_file = os.path.join(self.test_env_dir, 'test.css')
        helpers.create_test_files(path_to_test_file)
        resources = Resource.find_all_of_type_in_paths('javascript', test_env)
        self.assertEquals(None, resources)

    def test_exists_property(self):
        path_to_test_file = os.path.join(self.test_env_dir, 'test.js')
        helpers.clean_up_test_files(path_to_test_file)
        resource = Resource(path_to_test_file)
        # file does not exist yet
        self.assertFalse(resource.exists)
        helpers.create_test_files(path_to_test_file)
        self.assertTrue(resource.exists)
        helpers.clean_up_test_files(path_to_test_file)

    def test_content_property(self):
        path_to_test_file = os.path.join(self.test_env_dir, 'test.js')
        content = 'var foo = {};'
        helpers.create_test_file_with_content(path_to_test_file, content)
        resource = Resource(path_to_test_file)
        self.assertEquals(content, resource.content)
        helpers.clean_up_test_files(path_to_test_file)

    def test_requirements_property_for_resource_without_content_is_none(self):
        resource = Resource('some file')
        self.assertTrue(resource.content is None)
        self.assertEqual(None, resource.requirements)

    def test_requirements_property_for_resource_with_plain_content_is_none(self):
        path_to_test_file = os.path.join(self.test_env_dir, 'test.js')
        helpers.clean_up_test_files(path_to_test_file)
        content = 'var foo = {};'
        helpers.create_test_file_with_content(path_to_test_file, content)
        resource = Resource(path_to_test_file)
        self.assertTrue(resource.content == content)
        self.assertEqual(None, resource.requirements)
        helpers.clean_up_test_files(path_to_test_file)

    def test_javascript_requirements_are_found(self):
        path_to_test_file = os.path.join(self.test_env_dir, 'test.js')
        helpers.clean_up_test_files(path_to_test_file)
        content = '//= require jquery\nvar foo = {};//= require openlayers\n var s = some other thing\n'
        helpers.create_test_file_with_content(path_to_test_file, content)
        resource = Resource(path_to_test_file)
        self.assertTrue(resource.content == content)
        self.assertEqual(2, len(resource.requirements))
        self.assertEqual('jquery', resource.requirements[0].name)
        self.assertEqual((0,19), resource.requirements[0].insert_location)
        self.assertEqual('openlayers', resource.requirements[1].name)
        self.assertEqual((32,55), resource.requirements[1].insert_location)
        helpers.clean_up_test_files(path_to_test_file)

    def test_css_import_statements_found_as_requirements(self):
        path_to_test_file = os.path.join(self.test_env_dir, 'test.css')
        helpers.clean_up_test_files(path_to_test_file)
        content = 'h1 {background:red;}\n @import url("something.css")'
        helpers.create_test_file_with_content(path_to_test_file, content)
        resource = Resource(path_to_test_file)
        self.assertTrue(resource.content == content)
        self.assertEqual(1, len(resource.requirements))
        self.assertEqual('something', resource.requirements[0].name)
        self.assertEqual((22,50), resource.requirements[0].insert_location)

    def test_css_require_comments_found_as_requirements(self):
        path_to_test_file = os.path.join(self.test_env_dir, 'test.css')
        helpers.clean_up_test_files(path_to_test_file)
        content = 'h1 {background:red;}\n /*= require something  */'
        helpers.create_test_file_with_content(path_to_test_file, content)
        resource = Resource(path_to_test_file)
        self.assertTrue(resource.content == content)
        self.assertEqual(1, len(resource.requirements))
        self.assertEqual('something', resource.requirements[0].name)
        self.assertEqual((21,47), resource.requirements[0].insert_location)

    def test_merge_requirements_in_global_path(self):
        paths_to_test_files = [
            os.path.join(self.test_env_dir, 'dir1', 'file1.js'),
            os.path.join(self.test_env_dir, 'dir2', 'file2.js')]
        helpers.clean_up_test_files(paths_to_test_files)
        helpers.create_test_file_with_content(paths_to_test_files[0], '// This is file 1\n//= require FILE2')
        helpers.create_test_file_with_content(paths_to_test_files[1], '// This is file 2')
        file1_resource = Resource(paths_to_test_files[0])
        actual_merged_content = file1_resource.merge_requirements_from_paths(Paths(self.test_env_dir,
            include_cwd=False), previously_merged=[])

        self.assertEqual('// This is file 1\n// This is file 2', actual_merged_content)

        helpers.clean_up_test_files(paths_to_test_files)

    def test_merge_js_requirements_in_local_path(self):
        paths_to_test_files = [
            os.path.join(self.test_env_dir, 'dir1', 'file1.js'),
            os.path.join(self.test_env_dir, 'dir1', 'file2.js'),
            os.path.join(self.test_env_dir, 'dir2', 'file2.js')]
        helpers.clean_up_test_files(paths_to_test_files)
        helpers.create_test_file_with_content(paths_to_test_files[0], '// This is file 1\n//= require FILE2')
        helpers.create_test_file_with_content(paths_to_test_files[1], '// This is LOCAL file 2')
        helpers.create_test_file_with_content(paths_to_test_files[2], '// This is GLOBAL file 2')
        file1_resource = Resource(paths_to_test_files[0])
        actual_merged_content = file1_resource.merge_requirements_from_paths(Paths(self.test_env_dir,
                                                                                               include_cwd=False), previously_merged=[])

        self.assertEqual('// This is file 1\n// This is LOCAL file 2', actual_merged_content)

        helpers.clean_up_test_files(paths_to_test_files)


    def test_merge_js_requirements_favors_closer_files(self):
        paths_to_test_files = [
            os.path.join(self.test_env_dir, 'dir1', 'file1.js'),
            os.path.join(self.test_env_dir, 'dir1', 'file2.js'),
            os.path.join(self.test_env_dir, 'dir1', 'dir2', 'file2.js')]
        helpers.clean_up_test_files(paths_to_test_files)
        helpers.create_test_file_with_content(paths_to_test_files[0], '// This is file 1\n//= require FILE2')
        helpers.create_test_file_with_content(paths_to_test_files[1], '// This is NEARBY file 2')
        helpers.create_test_file_with_content(paths_to_test_files[2], '// This is FAR AWAY file 2')
        file1_resource = Resource(paths_to_test_files[0])
        actual_merged_content = file1_resource.merge_requirements_from_paths(Paths(self.test_env_dir,
            include_cwd=False), previously_merged=[])

        self.assertEqual('// This is file 1\n// This is NEARBY file 2', actual_merged_content)

    def test_merge_js_requirements_favors_downward_matches(self):
        paths_to_test_files = [
            os.path.join(self.test_env_dir, 'dir1', 'dir2', 'file1.js'),
            os.path.join(self.test_env_dir, 'dir1', 'file2.js'),
            os.path.join(self.test_env_dir, 'dir1', 'dir2', 'dir3', 'file2.js')]
        helpers.clean_up_test_files(paths_to_test_files)
        helpers.create_test_file_with_content(paths_to_test_files[0], '// This is file 1\n//= require FILE2')
        helpers.create_test_file_with_content(paths_to_test_files[1], '// This is UPWARD file 2')
        helpers.create_test_file_with_content(paths_to_test_files[2], '// This is DOWNWARD file 2')
        file1_resource = Resource(paths_to_test_files[0])
        actual_merged_content = file1_resource.merge_requirements_from_paths(Paths(self.test_env_dir,
            include_cwd=False), previously_merged=[])

        self.assertEqual('// This is file 1\n// This is DOWNWARD file 2', actual_merged_content)

    def test_merge_js_requirements_favors_shallow_downward_matches(self):
        paths_to_test_files = [
            os.path.join(self.test_env_dir, 'dir1', 'file1.js'),
            os.path.join(self.test_env_dir, 'dir1', 'dir2', 'file2.js'),
            os.path.join(self.test_env_dir, 'dir1', 'dir2', 'dir3', 'file2.js')]
        helpers.clean_up_test_files(paths_to_test_files)
        helpers.create_test_file_with_content(paths_to_test_files[0], '// This is file 1\n//= require FILE2')
        helpers.create_test_file_with_content(paths_to_test_files[1], '// This is SHALLOW file 2')
        helpers.create_test_file_with_content(paths_to_test_files[2], '// This is DEEP file 2')
        file1_resource = Resource(paths_to_test_files[0])
        actual_merged_content = file1_resource.merge_requirements_from_paths(Paths(self.test_env_dir,
            include_cwd=False), previously_merged=[])

        self.assertEqual('// This is file 1\n// This is SHALLOW file 2', actual_merged_content)

    def test_merge_js_requirements_favors_shallow_upward_matches(self):
        paths_to_test_files = [
            os.path.join(self.test_env_dir, 'dir1', 'dir2', 'dir3', 'file1.js'),
            os.path.join(self.test_env_dir, 'dir1', 'dir2', 'file2.js'),
            os.path.join(self.test_env_dir, 'dir1', 'file2.js')]
        helpers.clean_up_test_files(paths_to_test_files)
        helpers.create_test_file_with_content(paths_to_test_files[0], '// This is file 1\n//= require FILE2')
        helpers.create_test_file_with_content(paths_to_test_files[1], '// This is NEAR file 2')
        helpers.create_test_file_with_content(paths_to_test_files[2], '// This is FAR file 2')
        file1_resource = Resource(paths_to_test_files[0])
        actual_merged_content = file1_resource.merge_requirements_from_paths(Paths(self.test_env_dir,
            include_cwd=False), previously_merged=[])
        self.assertEqual('// This is file 1\n// This is NEAR file 2', actual_merged_content)

    def test_merge_css_requirements_in_local_path(self):
        paths_to_test_files = [
            os.path.join(self.test_env_dir, 'dir1', 'file1.css'),
            os.path.join(self.test_env_dir, 'dir1', 'file2.css'),
            os.path.join(self.test_env_dir, 'dir2', 'file2.css')]
        helpers.clean_up_test_files(paths_to_test_files)
        helpers.create_test_file_with_content(paths_to_test_files[0], '/* This is file 1 */\n@import url("FiLe2.css")')
        helpers.create_test_file_with_content(paths_to_test_files[1], '/* This is LOCAL file 2 */')
        helpers.create_test_file_with_content(paths_to_test_files[2], '/* This is GLOBAL file 2 */')
        file1_resource = Resource(paths_to_test_files[0])
        actual_merged_content = file1_resource.merge_requirements_from_paths(Paths(self.test_env_dir,
                                                                                               include_cwd=False), previously_merged=[])

        self.assertEqual('/* This is file 1 */\n/* This is LOCAL file 2 */', actual_merged_content)

        helpers.clean_up_test_files(paths_to_test_files)

    def test_merge_recursive_requirements_in_global_path(self):
        paths_to_test_files = [
                os.path.join(self.test_env_dir, 'dir1', 'file1.js'),
                os.path.join(self.test_env_dir, 'dir2', 'file2.js'),
                os.path.join(self.test_env_dir, 'dir3', 'file3.js')]
        helpers.clean_up_test_files(paths_to_test_files)
        helpers.create_test_file_with_content(paths_to_test_files[0], '// This is file 1\n//= require FILE2')
        helpers.create_test_file_with_content(paths_to_test_files[1], '// This is file 2\n//= require file3')
        helpers.create_test_file_with_content(paths_to_test_files[2], '// This is file 3')
        file1_resource = Resource(paths_to_test_files[0])
        actual_merged_content = file1_resource.merge_requirements_from_paths(Paths(self.test_env_dir, include_cwd=False), previously_merged=[])
        self.assertEqual('// This is file 1\n// This is file 2\n// This is file 3', actual_merged_content)
        helpers.clean_up_test_files(paths_to_test_files)

    def test_merge_from_global_path_does_not_add_requirements_twice(self):
        paths_to_test_files = [
            os.path.join(self.test_env_dir, 'dir11', 'file1.js'),
            os.path.join(self.test_env_dir, 'dir21', 'file2.js'),
            os.path.join(self.test_env_dir, 'dir31', 'file3.js'),
            os.path.join(self.test_env_dir, 'dir41', 'file4.js')]
        helpers.clean_up_test_files(paths_to_test_files)
        helpers.create_test_file_with_content(paths_to_test_files[0], '// This is file 1\n//= require file2\n//= require file3')
        helpers.create_test_file_with_content(paths_to_test_files[1], '// This is file 2\n//= require file4')
        helpers.create_test_file_with_content(paths_to_test_files[2], '// This is file 3\n//= require file4')
        helpers.create_test_file_with_content(paths_to_test_files[3], '// This is file 4\n')

        file1_resource = Resource(paths_to_test_files[0])
        actual_merged_content = file1_resource.merge_requirements_from_paths(Paths(self.test_env_dir, include_cwd=False), previously_merged=[])
        self.assertEqual('// This is file 1\n// This is file 2\n// This is file 4\n// This is file 3\n', actual_merged_content)
        helpers.clean_up_test_files(paths_to_test_files)

    def test_merge_library_required_at_the_top_of_multiple_files(self):
        paths_to_test_files = [
            os.path.join(self.test_env_dir, 'dir11', 'file1.js'),
            os.path.join(self.test_env_dir, 'dir21', 'file2.js'),
            os.path.join(self.test_env_dir, 'dir31', 'file3.js'),
            os.path.join(self.test_env_dir, 'dir41', 'file4.js')]
        helpers.clean_up_test_files(paths_to_test_files)
        helpers.create_test_file_with_content(paths_to_test_files[0], '//= require file2\n//= require file3\n// This is file 1\n')
        helpers.create_test_file_with_content(paths_to_test_files[1], '//= require file4\n// This is file 2\n')
        helpers.create_test_file_with_content(paths_to_test_files[2], '//= require file4\n// This is file 3\n')
        helpers.create_test_file_with_content(paths_to_test_files[3], '// This is file 4\n')

        file1_resource = Resource(paths_to_test_files[0])
        actual_merged_content = file1_resource.merge_requirements_from_paths(Paths(self.test_env_dir, include_cwd=False), previously_merged=[])
        self.assertEqual('// This is file 4\n// This is file 2\n// This is file 3\n// This is file 1\n', actual_merged_content)
        helpers.clean_up_test_files(paths_to_test_files)

    def test_find_all_in_paths(self):
        paths_to_processable_test_files = [
            os.path.join(self.test_env_dir, 'dir1', 'file1.js'),
            os.path.join(self.test_env_dir, 'dir2', 'file2.css')]

        paths_to_unprocessable_test_files = [
            os.path.join(self.test_env_dir, 'dir1', 'movie.avi'),
            os.path.join(self.test_env_dir, 'some.other.thing')]

        helpers.create_test_files(paths_to_processable_test_files)
        helpers.create_test_files(paths_to_unprocessable_test_files)

        resources = Resource.find_all_in_paths(Paths(self.test_env_dir, include_cwd=False))

        self.assertEquals(2, len(resources))
        self.assertEqual(paths_to_processable_test_files[0], resources[0].path_to_file)
        self.assertEqual(paths_to_processable_test_files[1], resources[1].path_to_file)

    def test_detects_minification_from_file_name(self):
        test_file_paths = [
            os.path.join(self.test_env_dir, 'dir1', 'file1.js'),
            os.path.join(self.test_env_dir, 'dir2', 'file1.min.js'),
            os.path.join(self.test_env_dir, 'dir2', 'file1-min.js')]

        helpers.create_test_files(test_file_paths)

        unminified_resource = Resource(test_file_paths[0])
        minified_resource_with_dot = Resource(test_file_paths[1])
        minified_resource_with_dash = Resource(test_file_paths[2])

        self.assertFalse(unminified_resource.minified)
        self.assertTrue(minified_resource_with_dot.minified)
        self.assertTrue(minified_resource_with_dash.minified)


# By Justin Walgran
# Copyright (c) 2012 Azavea, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import unittest
import os
import shutil
import tempfile
from helpers import create_test_file_with_content
from blend import YUICompressorMinifier, Resource

class TestYUICompressorMinifier(unittest.TestCase):

    def setUp(self):
        self.test_env_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.test_env_dir)

    def make_an_empty_js_file(self):
        test_file_path = os.path.join(self.test_env_dir, 'test.js')
        create_test_file_with_content(test_file_path, '')
        return test_file_path

    def make_a_js_file(self, content='var answer = 42;'):
        test_file_path = os.path.join(self.test_env_dir, 'test.js')
        create_test_file_with_content(test_file_path, content)
        return test_file_path

    def make_a_minified_js_file(self, separator='.'):
        test_file_path = os.path.join(self.test_env_dir, 'test%smin.js' % separator)
        create_test_file_with_content(test_file_path, 'var a=42;')
        return test_file_path

    def test_analysis_fails_when_lib_dir_is_not_found(self):
        invalid_lib_path = '/some/invalid/path'
        yuic = YUICompressorMinifier(lib_path=invalid_lib_path)
        test_resource = Resource(self.make_a_js_file())
        minification = yuic.minify(test_resource)
        self.assertFalse(minification.good, 'Expected the minification to be bad since the tools could not be found')
        self.assertEquals(1, len(minification.errors))
        self.assertEquals('A YUI Compressor .jar file could not be found in %s.' %  invalid_lib_path, minification.errors[0])

    def test_minifying_an_empty_resource_returns_empty_minification_content(self):
        test_resource = Resource(self.make_an_empty_js_file())
        yuic = YUICompressorMinifier()
        minification = yuic.minify(test_resource)
        self.assertTrue(minification.good)
        self.assertEqual('', minification.content)

    def test_minifying_an_already_minified_resource_returns_a_message_and_unmodified_content(self):
        test_resource = Resource(self.make_a_minified_js_file())
        self.assertTrue(test_resource.minified)
        yuic = YUICompressorMinifier()
        minification = yuic.minify(test_resource)
        self.assertTrue(minification.good)
        self.assertEqual(test_resource.content, minification.content)
        self.assertEqual('The resource %s is already minified.' % test_resource.path_to_file,
            minification.errors_warnings_and_messages_as_string)

    def test_compressor(self):
        test_resource = Resource(self.make_a_js_file(
            content='var answer = 42;\nvar question = "what is " +\n "6 times 7";'))
        yuic = YUICompressorMinifier()
        minification = yuic.minify(test_resource)
        self.assertTrue(minification.good)
        self.assertEqual('var answer=42;var question="what is 6 times 7";', minification.content)

    def test_compressor_failure(self):
        test_resource = Resource(self.make_a_js_file(
            content='var obj = { function: "failure" }')) # 'function' is not a legal property name
        yuic = YUICompressorMinifier()
        minification = yuic.minify(test_resource)
        self.assertFalse(minification.good)


# By Justin Walgran
# Copyright (c) 2012 Azavea, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

class Result(object):

    def __init__(self):
        self._messages = None
        self._warnings = None
        self._errors = None
        self._good = False

    @property
    def messages(self):
        """
        A list of informational topics generated by analysing some content
        """
        return self._messages

    @property
    def warnings(self):
        """
        A list of warning topics generated by analysing some content
        """
        return self._warnings

    @property
    def errors(self):
        """
        A list of warning topics generated by analysing some content
        """
        return self._errors

    @property
    def good(self):
        """
        A boolean that indicates whether or not the analyzed content is 'good'
        """
        return self._good

    @property
    def errors_warnings_and_messages_as_string(self):
        output = None
        if self.errors:
            output = '\n'.join(self.errors)

        if self.warnings:
            if output:
                output += '\n' + '\n'.join(self.warnings)
            else:
                output = '\n'.join(self.warnings)

        if self.messages:
            if output:
                output += '\n' + '\n'.join(self.messages)
            else:
                output = '\n'.join(self.messages)

        return output

    def add_message(self, message):
        if message is not None:
            if self._messages is None:
                self._messages = []
            self._messages.append(message)

    def add_messages(self, message_list):
        if message_list is not None:
            if self._messages is None:
                self._messages = []
            self._messages.extend(message_list)

    def add_warning(self, warning):
        if warning is None:
            return

        if self._warnings is None:
            self._warnings = []
        self._warnings.append(warning)

    def add_error(self, error):
        if error is None:
            return

        if self._errors is None:
            self._errors = []
        self._errors.append(error)

    def mark_as_good(self):
        self._good = True

    def mark_as_bad(self):
        self._good = False

    def __str__(self):
        return self.__unicode__()

    def __unicode__(self):
        return self.errors_warnings_and_messages_as_string



# By Justin Walgran
# Copyright (c) 2012 Azavea, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import os
import sys
import subprocess
from distutils.core import setup, Command
import setuptools

def read(filename):
    return open(os.path.join(os.path.dirname(__file__), filename)).read()

class RunTests(Command):
    description = "Run the unit test suite for blend."
    user_options = []
    extra_env = {}
    extra_args = []

    def run(self):
        run_tests_script_path = os.path.join(os.path.dirname(__file__), 'blend', 'run_tests.py')
        sys.exit(subprocess.call([run_tests_script_path]))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

setup(
    name='Blend',
    version='0.0.1',
    author='Justin Walgran',
    author_email='jwalgran@azavea.com',
    packages=['blend', 'blend.test'],
    scripts=['bin/blend'],
    package_data={'blend': [
        'lib/js.jar',
        'lib/js_LICENSE.TXT',
        'lib/jslint.js',
        'lib/yuicompressor-2.4.6.jar',
        'lib/yuicompressor_LICENSE.TXT'
    ]},
    url='http://github.com/azavea/blend',
    license='LICENSE.txt',
    description='A cross-platform tool for merging and processing client-side assets for a web application.',
    long_description=read('README.rst'),
    cmdclass = { 'test': RunTests },
    keywords = "javascript css html build",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Topic :: Software Development :: Build Tools"
    ]
)



# By Justin Walgran
# Copyright (c) 2012 Azavea, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import unittest
from blend import Minification

class TestMinification(unittest.TestCase):

    def setUp(self):
        self.minification = Minification()

    def tearDown(self):
        pass

    def test_has_a_content_property(self):
        self.assertIsNone(self.minification.content)

    def test_can_set_content(self):
        self.minification.set_content("content")
        self.assertEqual("content", self.minification.content)

    def test_converting_a_good_minification_to_string_returns_content(self):
        self.minification.mark_as_good()
        self.minification.set_content("content")
        self.assertEqual("content", str(self.minification))

    def test_converting_a_bad_minification_to_string_returns_error(self):
        self.minification.mark_as_bad()
        self.minification.set_content("content")
        self.minification.add_error("error message")
        self.assertEqual("error message", str(self.minification))